/**
 * Copyright (c) Istituto Nazionale di Fisica Nucleare, 2006-2014.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.italiangrid.voms.clients.impl;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.italiangrid.voms.request.impl.DefaultVOMSESLookupStrategy;

public class CustomVOMSESLookupStrategy extends DefaultVOMSESLookupStrategy {

  String customVOMSESPath;

  public CustomVOMSESLookupStrategy(String vomsesPath) {

    this.customVOMSESPath = vomsesPath;
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.italiangrid.voms.request.impl.BaseVOMSESLookupStrategy#lookupVomsesInfo
   * ()
   */
  @Override
  public List<File> lookupVomsesInfo() {

    List<String> searchedPaths = searchedPaths();
    List<File> vomsesPaths = new ArrayList<File>();

    for (String p : searchedPaths) {
      File f = new File(p);
      if (f.exists())
        vomsesPaths.add(f);
    }

    return vomsesPaths;
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.italiangrid.voms.request.impl.BaseVOMSESLookupStrategy#searchedPaths()
   */
  @Override
  public List<String> searchedPaths() {

    List<String> searchedPaths = super.searchedPaths();
    searchedPaths.add(customVOMSESPath);
    return searchedPaths;
  }

}
