/**
 * Copyright (c) Istituto Nazionale di Fisica Nucleare, 2006-2014.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.italiangrid.voms.clients.impl;

import java.util.List;

import eu.emi.security.authn.x509.proxy.ProxyCertificate;

/**
 * A listener which informs of succesfull creation of a VOMS proxy certificate
 * 
 * @author andreaceccanti
 *
 */
public interface ProxyCreationListener {

  /**
   * Informs of the succesfull creation of a VOMS proxy certificate
   * 
   * @param proxyPath
   *          the file where the proxy has been saved
   * @param proxy
   *          the {@link ProxyCertificate}
   * @param warnings
   *          a list of warnings related to this proxy creation
   */
  public void proxyCreated(String proxyPath, ProxyCertificate proxy,
    List<String> warnings);

}
